/* configEncoder.h */
#ifndef CONFIG_GHS_ENCODER_H__
#define CONFIG_GHS_ENCODER_H__
#include "GhsControlStructs.h"


/*
 * How to use this library
 * A PHD consists of three basic parts
 *   1. Time and clock information - needed if the PHD stores data or reports time stamps in the measurements
 *   2. Information about itself, so-called system info; All PHDs have this
 *   3. The measurements
 * This library allows you to configure the above parts for your device and generate a byte array that is to be
 * sent when needed. The measurement byte array is then updated as you receive measurement data from the sensor
 * with the new measurement values and timestamp (if you are including time stamps). In this standard measurements
 * are grouped by time stamp. For example, a blood pressure cuff sending both blood pressure and pulse rate usually
 * sends the blood pressure and pulse rate with the same time stamp, and if you want to do that, you add both the
 * blood pressure and pulse rate to the same group. So on the wire the measurement would look as follows:
 *               | HEADER WITH TIME STAMP | BLOOD PRESSURE | PULSE RATE |
 * The library creates the byte array above and then provides methods to update that byte array with new values from
 * your sensor and appropriate time stamp. The work flow would be as follows:
 *    On powerup you would first create the time properties and measurement group:
        s_MsmtGroup *msmtGroup = NULL;              // Initialize the pointer to the measurement group to NULL. Setting to NULL is important!
        unsigned char group_id = 0;
        result = createMsmtGroup(&msmtGroup,        // The library will populate this structure for you. It will allocate memory for it
                                                    // which needs to be cleaned up. The cleanup method will set the pointer back to NULL.
                                                    // If the parameter is not NULL, this method will attempt to free it which may cause
                                                    // a crash if it has not yet been allocated! NOTE THAT IT IS A POINTER to a pointer that
                                                    // gets passed.
                                 true,              // If time stamps are being used set to true
                                 2,                 // The number of measurements in the group, in this case 2, blood pressure and pulse rate
                                 group_id++);       // The group id. This value is only used for optimization should you choose to use optimization
                                                    // but it needs to be valued and unique for the group. There is a maximum of 256 groups.
 *    Now we need to add measurements to that group. The blood pressure is more complicated because it is a compound or a vector. The systolic,
 *    diastolic, and mean components are treated as parts of a single measurement like the x, y, and z components of an acceleration. Thus before
 *    we can create this measurement we need to specify what our compound measurement contains. That is done by populating an array of s_Compound
 *    structs as follows:
        s_Compound compounds[3];            // We want three; one each for the systolic, diastolic, and mean components:
        compounds[0].subType = MDC_PRESS_BLD_NONINV_SYS;    // This is a 32-bit unsigned integer which gives the MDC code for systolic blood pressure
        compounds[1].subType = MDC_PRESS_BLD_NONINV_DIA;
        compounds[2].subType = MDC_PRESS_BLD_NONINV_MEAN;
 *    Yes it is understood that you have to know what codes are and that is not always a trivial task. Included with this library is a file nomenclature.h
 *    which contains a bunch of MDC codes that will cover 99% of the possible PHDs one can create. There is also an on line tool called the NIST Rosetta
 *    which can also help though at this time it is still a work in progress and the nomenclature.h file is your better bet.
 *    The other element of this structure is the value. That we will set when we get data from the sensor. The subTypes can be set once as they will
 *    never change. So once we decide what is in our compound measurement, we create the compound measurement and then add it to our group:
        s_GhsMsmt *bp = NULL;               // Initialize this pointer to NULL!
        result = createCompoundNumericMsmt(&bp,                     // The library will populate this structure. It will allocate memory for it
                                                                    // and attempt to free it if it is not NULL which may cause a crash if it has
                                                                    // not been previously allocated. NOTE THAT IT IS A POINTER to a pointer that
                                                                    // gets passed.
                                           MDC_PRESS_BLD_NONINV,    // This is the 32-bit MDC code that expresses what the overall compound measurement
                                                                    // is. IN this case it is 'non-invasive blood pressure'.
                                           true,                    // true If the values are encoded as two-byte Mder Floats versus 4-byte MderFloats
                                           MDC_DIM_MMHG,            // The units of the measurement. At this time the GHS standard does not support
                                                                    // different units for each subType of the compound. If support is added, the units
                                                                    // will be part of the compound structure.
                                           3,                       // The number of elements in the compound. In this case, there are three.
                                           compounds);              // A pointer to the array of s_Compound structs.
 *    Many blood pressure cuffs do not do this but we want to tell the world that this blood pressure cuff is an upper arm cuff. In the GHS standard we
 *    can do this by including an extra MDC code in a field called 'supplemental types'. The Supplemental types allow one to say a lot of different
 *    things about the measurement, for example that it is an average, a maximum, that the sensor is placed on a certain location of the body, etc.
 *    To add a supplemental type to our measurement we create an array of unsiged long integers and populate it with the supplemental type codes we
 *    want to add.
        unsigned long supplementalTypes[1] = {MDC_UPEXT_ARM_UPPER};  // Our array of codes consists of only one entry.
        result = setGhsMsmtSupplementalTypes(&bp,                    // A pointer to the s_GhsMsmt pointer for the Blood Pressure we have previously created
                                             supplementalTypes,      // A pointer to the array of supplemental types to add
                                             1);  //                 // The size of that array
 *    Now we add this populated s_GhsMsmt struct to the group:
        bp_index = addGhsMsmtToGroup(bp,            // A pointer to the s_GhsMsmt struct we want to add - in this case the bp
                                     &msmtGroup);   // A pointer to the measurement group we want to add the measurement to.
 *    Note that the above method returns a very critical bp_index. You have to keep this value as it tell where in the measurement group this particular measurement
 *    is. We will discuss that later.
 *    Now lets create and add the pulse rate. The pulse rate is a simple numeric and is simpler to contruct. The vast majority of PHD measurements are simple
 *    numerics.
        s_GhsMsmt *pr = NULL;               // Again, Initialize this pointer to NULL!
        result = createNumericMsmt(&pr,                     // A pointer to our s_GhsMst pointer that will be populated by the library.
                                   MDC_PULS_RATE_NON_INV,   // MDC code for what this measurement is, in this case a non-invasive pulse rate. It tells 
                                                            // downstream readers that this value is not only a pulse rate but it was generated from
                                                            // a blood pressure cuff device.
                                   true,                    // true If the values are encoded as two-byte Mder Floats versus 4-byte MderFloats
                                   MDC_DIM_BEAT_PER_MIN);   // The units of the pulse rate which is beats per minute
        pr_index = addGhsMsmtToGroup(pr, &msmtGroup);       // Add the measurement to the group
 *    Note again the very important pr_index value. You need to save this!

 *    Now we have populated our structures for the bllod pressure/pulse rate device. But structures are not sent on the wire. SO the next step is to create
 *    the byte array that will be sent over the airwaves. ONce we have created this byte array, we do not need the structs we have populated. All we need
 *    is the byte array struct and the indices of the measurements in the group. So now lets create the byte array struct. This struct cannot go out of scope!
 *    It must exist for the duration of the connection! The byte array of this struct will be populated with sensor data and delivered to the PHG.
        s_MsmtGroupData *msmtGroupBpData                = NULL;     // Defined as a global variable or in some manner that it will not go out of scope

        result = createMsmtGroupDataArray(&msmtGroupBpData,     // Pointer to the s_MsmtGroupData pointer initialized to NULL. The library will populate this
                                                                // structure which contains the byte array. It will allocate memory for it
                                                                // and attempt to free it if it is not NULL which may cause a crash if it has
                                                                // not been previously allocated. NOTE THAT IT IS A POINTER to a pointer that gets passed.
                                          msmtGroup,            // The measurement group to encode into a byte array
                                          sGhsTime,             // A pointer to the s_GhsTime struct containing the time properties of the PHD
                                          PACKET_TYPE_NORMAL);  // For now, use this. There is also an optimization option which we will for the moment not use.
 *    Now that we have populated this struct, we dont need the s_MsmtGroup struct anymore. We call the cleanup method to free the allocated resources.
         cleanUpMsmtGroup(&msmtGroup); // cleans up any allocated data -  we only need the data array now
 *    Note that all of this can happen at power up, so there is no performance penalty while connected.

 *    How to use it?
 *    Assume we get data from the sensor of BP: 102, 69, 81, PR: 64  timestamp epoch counter. To enter these values into our data array we call a set of
 *    update methods. There is an update method for each measurement type. In this case there is one for the compound and one for the simple. So we call
 *    these methods and pass in the measurement values. Now many Blood pressure sensors provide data as simple integers. But other sensors report data
 *    that represent values that have fractional parts. The GHS standard requires that all numerical values be formatted in a manner that gives the precision
 *    of the data. Thus a value fo 2, 2.0, 2.00, and 2.000 are all 'different'. They are all numerically equivalent but it is clear that one sensor reports
 *    the data at much greater precision than another. To do this, the data is formatted in what is called IEEE 20601 Mder SFLOATs and FLOATs. In this case
 *    we are using two-byte SFLOATs. So we need to encode our three BP values into three s_MderFloat structs
 *
        void dataFromSensor(short int systolic, short int diastolic, short int mean, short int pulseRate)  // Get data from sensor
        {
            s_MderFloat mder[3];                    // One MderFloat struct for each BP component
            mder[0].mderFloatType = MDER_SFLOAT;    // We are using 2-byte SFLOATs
            mder[0].specialValue = MDER_NUMBER;     // Just a regular number, not a NAN (Not a number) or a PINF (positive infinity) or NINF (negative infinity)
            mder[0].exponent = 0;                   // Exponent is 0. That means there are no diciaml fraction. Precision is integer level 0.
            mder[0].mantissa = systolic;            // The manitissa is the integer, for example the above 102 -  if one set exponent to -1 and mantissa to 1020,
                                                    // the result would be 102.0
            mder[1].mderFloatType = MDER_SFLOAT;
            mder[1].specialValue = MDER_NUMBER;
            mder[1].exponent = 0;
            mder[1].mantissa = diastolic;

            mder[2].mderFloatType = MDER_SFLOAT;
            mder[2].specialValue = MDER_NUMBER;
            mder[2].exponent = 0;
            mder[2].mantissa = mean;

            updateDataCompound(&msmtGroupBpData,    // Here is the structure containing the data array
                               bp_index,            // This is the index to the BP measurement in the group
                               mder,                // A pointer to the array of s_MderFloat structs
                               msmt_id++);          // The measurement instance number.
                                                    // Reuse the first element of the s_MderFloat structs
            mder[0].mderFloatType = MDER_SFLOAT;    // We are using 2-byte SFLOATs
            mder[0].specialValue = MDER_NUMBER;     // Just a regular number, not a NAN (Not a number) or a PINF (positive infinity) or NINF (negative infinity)
            mder[0].exponent = 0;                   // Exponent is 0. That means there are no diciaml fraction. Precision is integer level 0.
            mder[0].mantissa = pulseRate;           // Now this is the only one that is really different in this case
                                                    // Now populate the pulse rate of the data array
            updateDataNumeric(&msmtGroupBpData,     // Here is the structure containing the data array
                              pr_index,             // Critical index to the pulse rate measurement in the group
                              &mder[0],             // The value to update
                              msmt_id++);           // The instance of this measurement.
            // Now populate the time stamp
            updateTimeStampEpoch(&msmtGroupBpData,  // Here is the structure containing the data array
                                 epoch);            // The updated current epoch time counter
            // There is one more step. We need to add the command given by the PHG that we are responding to. It can only be one of two possible values
            // Command to get stored data or command to get live data. That would be done as follows:
            msmtGroupBpData->data[0] = (current_command & 0xFF);
            msmtGroupBpData->data[1] = ((current_command >> 8) & 0xFF);
       }
 * The data array has been populated and is now ready to notify over the characteristic. The actual data array is 
       msmtGroupBpData->data

 * Now in practice there are going to be few PHDs that include all that configuration code. Instead, they can create and populate that s_MsmtGroupData struct 
 * and the measurement indices and include only that in the code, storing it in ROM. One can go a step further and create just the data array and the indices
 * to the locations in the data array for updates and not have the update routines to contend with. It's a bit more work though.
 *
 * A similar approach is used to configure the the sGhsTime properties, the current time info and the system info.

 */

/**
 */
void cleanUpSystemInfo(s_SystemInfo** systemInfo);

void cleanUpSystemInfoData(s_SystemInfoData** systemInfoData);

#if (USES_TIMESTAMP == 1)
    void cleanUpGhsTime(s_GhsTime** sGhsTime);
#endif
void cleanUpTimeInfo(s_TimeInfo** timeInfo);

/**
 * Call this at the end of the program. Frees up all the allocated resources use in creating the TimeInfo Data
 * @param timeInfoData pointer a pointer of the application's s_TimeInfoData struct.
 */
void cleanUpTimeInfoData(s_TimeInfoData** timeInfoData);

/**
 * This standard defines a current time and time stamp that contain a base epoch, a set of flags, a dst shift an offset shift,
 * and a time sync. In binary form it is 10 bytes long. The set of flags indicate whether the resolution of the clock is
 * milliseconds or seconds. PHDs that require different resolutions cannot be supported by this standard.

 * Most PHDs are not able to support knowledge the DST (daylight savings time) shift or Offset shift from UTC to local time as
 * this knowledge requires the PHD to link to some external time source or a GPS which for cost reasons is not done. The drawback
 * is that such PHDs can only resport measurements from a single time zone. However, since the epoch is UTC, a PHG can always
 * convert the epoch to a correct civil time in the time zone of the PHG. For example, if a measurement was taken two time zones
 * to the east at 14:00 in that time zone but uploaded in the current time zone the civil time would be reported at the corresponding
 * civil time in the current time zone which would be 12:00.
 * The dst/offsetShift are only changed if the PHD supports knowledge of these values. If the PHD does not have knowledge
 * of these values, special values are used to indicate the lack of support for DST and Offset. Such a PHD will always send its time
 * values with indicators showing that it does not know these values.
 * Since most PHDs do not support an independent means of syncrhonizing electronically to an external time source they either let the
 * user set the clock or the PHG. If the user sets the clock the PHD shall set the timeSync to MDC_TIME_SYNC_EBWW (eyeball and wrist
 * watch). If the PHG has previously set the time in an earlier connection, the PHD sets the time Sync to the value provided by the
 * PHG instead. If the PHD has not been synchronized, it sets the value to MDC_TIME_SYNC_NONE.
 * 
 * This method creats and populates the s_GhsTime structure with the initial properties to create and update the time fields in the
 * the current time and measurement time stamps.
 *
 * @param sGhsTime pointer to an s_GhsTime pointer. This method will allocate memory for the structure and populate it. It is important
 *        that the pointer itself is initialized to NULL the first time it is called. In other words
 *           s_GhsTime *sGhsTime = NULL;        // Set to NULL 
 *           createGhsTime(&sGhsTime, ...);     // Populate the structure
 *                some stuff
 *           createGhsTime(&sGhsTime, ...);     // Okay to reuse same variable - call will free if not NULL before repopulating
 *                some stuff
 *           cleanUpGhsTime(&sGhsTime);         // clean up allocated resources before program exit.
 *        All 'create' methods work as above.
 * @param offsetShift the current Offset to local time from UTC in units of 15 minutes IF the PHD is able to obtain this information.
 *        Allowable range is -12 to + 12. All else is taken as unsupported.
 *        If the PHD does not have the ability to obtain this information, set the value to GHS_TIME_OFFSET_UNSUPPORTED. If set to 
 *        GHS_TIME_OFFSET_UNSUPPORTED this is a fixed property of the PHD and cannot change (most PHDs).
 * @param supportsMilliseconds true if the clock tick resolution is in milliseconds, otherwise the clock resolution is seconds. The
 *        epoch time stamp is always reported as milliseconds, but in a seconds-resolution device, the last three digits will always
 *        be 000.
 * @param timeSync the current state of timeSynchronization. This may be updated by a PHG set time.
 * @return true if all went well. false indicates bad input parameters or unable to allocate memory
 */
#if (USES_TIMESTAMP == 1)
    bool createGhsTime(s_GhsTime **sGhsTime, short offsetShift, bool supportsMilliseconds, unsigned short timeSync);
#endif
/**
 * Sets up the time info for the device. Method allocates memory for the s_TimeInfo.
 * Only one time clock type can exist. It is also possible to support no time clock at all such as for continuous live measurements.

 * @param timeInfo pointer to an s_TimeInfo *struct pointer. This method will populate it. The pointer must remain in scope
 * @param sGhsTime a pointer to s_GhsTime struct created by the createGhsTime method. Set to NULL if there are no time stamps in any measurements
 * @param timeSupport one of the following possible values:
 *       TIME_FLAGS_NO_TIME        PHD does not report time stamps in measurements or store measurements
 *       TIME_FLAGS_RELATIVE_TIME  PHD supports an 'arbitrary' or relative time. Just a tick counter where the 0 value has no semantic meaning
 *       TIME_FLAGS_EPOCH_TIME     PHD supports just the UTC epoch but does not understand DST or Offsets. This is equivalent to ABS time
 *       TIME_FLAGS_EPOCH_OFFET    PHD supports UTC epoch plus knows UTC offset to local time (perhaps via GPS)
 *       TIME_FLAGS_BO_TIME        PHD supports full base offset time (epoch, dst shift, and offset to local time from UTC).
 * @param allowSetTime true if PHD clock can be set. Applies only to absolute and base offset times. Ignored otherwise
 * @returns true if there are no problems.
 */
bool createTimeInfo(s_TimeInfo **sTimeInfo, s_GhsTime *sGhsTime, unsigned short timeSupport, bool allowSetTime);

/**
 * This method generates the time info byte array to be sent to the client when the client asks for it. The only thing that will
 * be missing is the actual current time. The application will have to fill in the current time value appropriate for the time clock
 * type when the client asks for the current time info. The index of the current time in the byte array is given by timeInfoDataPtr->currentTime_index.
 * How those 8 bytes are filled depends upon the time stamp type. For relative times it is a 64-bit unsigned long. Remember to populate the 
 * 8 bytes little endian. For absolute time it is 8 individual bytes as century, year, month, etc. in that order.
 * The next time this array will need to be messed with is if the client sets the time. The application will need to update it's clock
 * as approriate and it is not really necessary to update the current time value in this array until the clients asks for this info again.
 * However, the set time also comes with a time sych method and that will need to be updated. The index of the time sync in the byte array is
 * given by timeInfoDataPtr->timeSync_index. That is a 16-bit unsigned short so it is little endian. (It arrives little endian of course.)
 * Helper routines to update the current time and to update the buffer on a client set-time sync are in the works.
 * @param timeInfoDataPtr pointer to an s_TimeInfoData *struct. This method will populate it. The pointer must remain in scope
 * @param timeInfo pointer to the sTimeInfo struct populated by the createTimeInfo method.
 * @returns true if there are no problems.
 */
bool createCurrentTimeDataBuffer(s_TimeInfoData** timeInfoData, s_TimeInfo *timeInfo);

/**
 * Updates the current time info data array from a PHG set time operation.
 * @param timeInfoDataPtr pointer to an s_TimeInfoData *struct. This method will uodate it. The pointer must remain in scope
 * @param update will be a 10-byte array from the set time operation.
 * @param length just a check.
 */
 #if (USES_TIMESTAMP == 1)
    bool updateCurrentTimeFromSetTime(s_TimeInfoData** timeInfoData, unsigned char *update, unsigned short length);
#endif
/**
 * Updates the current time info data array from the PHD (in response to a PHG get current time info command).
 * Epoch update is done alone in a second call as is time sync - this call is expected to be very rare
 * @param timeInfoDataPtr pointer to an s_TimeInfoData *struct. This method will uodate it. The pointer must remain in scope
 * @param offsetShift the current timezone offset shift from UTC in 15-minute units. If not supported set to 0xFF.
 */
bool updateCurrentTimeFromPhd(s_TimeInfoData** timeInfoData, short offsetShift);

bool updateCurrentTimeFromPhdTimeSync(s_TimeInfoData** timeInfoData, unsigned short timeSync);

bool updateCurrentTimeEpoch(s_TimeInfoData** timeInfoData, unsigned long long epoch);

unsigned short getTimeSync(s_TimeInfoData* timeInfoData);



bool createSystemInfo(s_SystemInfo **systemInfo, unsigned short numberOfSpecializations);

bool setSystemIdentifierByte(s_SystemInfo **systemInfo, unsigned char *systemId);

bool addSpecialization(s_SystemInfo **systemInfo, unsigned short specializationTermCode, unsigned short version);

bool setRequiredSystemInfoStrings(s_SystemInfo **systemInfo, char *manufacturerName, char *modelNumber);

bool setOptionalSystemInfoStrings(s_SystemInfo **systemInfo, char *serialNumber, char *firmware, char *hardware, char *software);

/**
 *  The set value 'isRegulated' is only used if 'hasRegulationStatus' is true.
 */
bool setRegulationStatus(s_SystemInfo **systemInfoPtr, bool hasRegulationStatus, bool isRegulated);

/**
 * 16-digit HEX string. This method will put the system id in little endian order as bytes.
 */
bool setSystemIdentifier(s_SystemInfo **systemInfo, char *systemId);

#if (USES_SYSTEM_AVAS == 1)
    bool initializeSystemInfoAvas(s_SystemInfo** systemInfo, unsigned short numberOfAvas);

    bool addSystemInfoAvas(s_SystemInfo** systemInfo, s_Avas *ava);
#endif

bool createSystemInfoData(s_SystemInfoData** systemInfoData, s_SystemInfo *systemInfo);


void cleanUpGhsMsmt(s_GhsMsmt** ghsMsmt);

void cleanUpMsmtGroup(s_MsmtGroup** msmtGroup);

void cleanUpMsmtGroupData(s_MsmtGroupData** msmtGroupData);

bool createMsmtGroup(s_MsmtGroup **msmtGroup, bool hasTimeStamp, unsigned char numberOfGhsMsmts, unsigned char groupId);

bool setHeaderOptions(s_MsmtGroup **msmtGroup, bool areSettings, bool hasPersonId, unsigned short personId);

bool setHeaderSupplementalTypes(s_MsmtGroup **msmtGroup, unsigned long *supplementalTypes, unsigned short numberOfSupplementalTypes );

bool setHeaderSrcHandleRefs(s_MsmtGroup **msmtGroup, unsigned short numberOfSrcHandleRefs );

bool setHeaderDuration(s_MsmtGroup **msmtGroup, s_MderFloat *duration );

#if (USES_HEADER_AVAS == 1)
    bool initializeHeaderAvas(s_MsmtGroup **msmtGroup, unsigned short numberOfAvas);

    bool addHeaderAva(s_MsmtGroup** msmtGroup, s_Avas* avaIn);
#endif

bool updateTimeStampEpoch(s_MsmtGroupData **msmtGroupDataPtr, unsigned long long epoch);
bool updateTimeStampTimeSync(s_MsmtGroupData **msmtGroupDataPtr, unsigned short timeSync);
bool updateTimeStampOffset(s_MsmtGroupData **msmtGroupDataPtr, short offsetShift);

short addGhsMsmtToGroup(s_GhsMsmt *ghsMsmt, s_MsmtGroup **msmtGroupPtr);

bool updateDataHeaderSrcHandleRef(s_MsmtGroupData** msmtGroupData, unsigned short ref, unsigned ref_index);
bool updateDataGhsMsmtSrcHandleRef(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, unsigned short ref, unsigned ref_index);

#if(USES_NUMERIC == 1)
    bool createNumericMsmt(s_GhsMsmt **ghsMsmt, unsigned long type, bool isSfloat, unsigned short units);

    bool updateNumericMsmt(s_GhsMsmt **ghsMsmt, s_MderFloat *value, unsigned short msmt_id);

    bool updateDataNumeric(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, s_MderFloat* value, unsigned short msmt_id);
#endif
#if(USES_COMPOUND == 1)
    bool createCompoundNumericMsmt(s_GhsMsmt** ghsMsmt, unsigned long type, bool isSfloat, unsigned short units, 
            unsigned short numberOfComponents, s_Compound *compounds);  // Classic compound
    bool createComplexCompoundNumericMsmt(s_GhsMsmt** ghsMsmt, unsigned long type, bool isSfloat, 
            unsigned short numberOfComponents, s_Compound *compounds);  // Complex compound

    bool updateCompoundMsmt(s_GhsMsmt **ghsMsmt, s_MderFloat *value, unsigned short msmt_id);

    bool updateDataCompound(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, s_MderFloat* value, unsigned short msmt_id);
#endif
#if(USES_CODED == 1)
    bool createCodedMsmt(s_GhsMsmt** ghsMsmt, unsigned long type);

    bool updateCodedEnum(s_GhsMsmt **ghsMsmt, unsigned long code, unsigned short msmt_id);

    bool updateDataCoded(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, unsigned long code, unsigned short msmt_id);
#endif
#if (USES_BITS16 == 1)
    bool createBit16EnumMsmt(s_GhsMsmt** ghsMsmt, unsigned long type, unsigned short state, unsigned short support);

    bool updateBits16Enum(s_GhsMsmt** ghsMsmt, unsigned short bits16, unsigned short msmt_id);

    bool updateDataBits16(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, unsigned short bits16, unsigned short msmt_id);
#endif

#if (USES_BITS32 == 1)
    bool createBit32EnumMsmt(s_GhsMsmt** ghsMsmt, unsigned long type, unsigned long state, unsigned long support);

    bool updateBits32Enum(s_GhsMsmt** ghsMsmt, unsigned long bits32, unsigned short msmt_id);

    bool updateDataBits32(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, unsigned long bits32, unsigned short msmt_id);
#endif

#if (USES_RTSA == 1)
    /**
     * @param sampleSize number of bytes per sample
     */
    bool createRtsaMsmt(s_GhsMsmt** ghsMsmt, unsigned long type, unsigned short units, s_MderFloat* period, s_MderFloat* scaleFactor, s_MderFloat* offset,
        unsigned short numberOfSamples, unsigned char sampleSize);

    bool updateRtsa(s_GhsMsmt** ghsMsmt, unsigned char* samples, unsigned short msmt_id);

    bool updateDataRtsa(s_MsmtGroupData** msmtGroupData, unsigned short msmtIndex, unsigned char* samples, unsigned short dataLength, unsigned short msmt_id);
#endif

bool setGhsMsmtSupplementalTypes(s_GhsMsmt **ghsMsmt, unsigned long *supplementalTypes, unsigned short numberOfSupplementalTypes );

bool setGhsMsmtSrcHandleRefs(s_GhsMsmt **ghsMsmt, unsigned short numberOfSrcHandleRefs );

bool setGhsMsmtDuration(s_GhsMsmt **ghsMsmt, s_MderFloat *duration );

#if (USES_MSMT_AVAS == 1)
    bool initializeGhsMsmtAvas(s_GhsMsmt **ghsMsmt, unsigned short numberOfAvas);

    bool addGhsMsmtAva(s_GhsMsmt **ghsMsmt, s_Avas *ava);
#endif

bool createMsmtGroupDataArray(s_MsmtGroupData** msmtGroupData, s_MsmtGroup *msmtGroup, s_GhsTime *sGhsTime, unsigned short packetType);

#endif  //CONFIG_GHS_ENCODER_H__
